#include "mbed.h"
#include <iostream>
#include <iomanip>
#include <string>

#include "24LCxx_I2C.h"

/*
 * Declare statics
 */
DigitalOut g_availableLed(p20); // To verify if program in running
Ticker g_available; // LED1 will flash with a period of 2s
C24LCXX_I2C eeprom(p9, p10, 0x01, NC, 400000); // Create an instance of the class C24LCXX_I2C, p9/28: SDA, p10/29:SDL, WP not managed, address: A3=0,A2=0,A1=1, on 3.3V I2C bus

/*
 * Declare functions
 */
void AvailableIndicator(); // LED1 flashing for program while program is alive

int main() {
    // Launch available indicator
    g_available.attach(&AvailableIndicator, 1.0);
    
    int counter;
    int maxCounter = 4;
    short memoryAddress = 0xff;
    std::cout << "Staring application v0.0.0.1, memory address: " << memoryAddress << "\r" << std::endl;
    // Test byte read/write
    unsigned char byteValue = 0xaa;
    eeprom.Write(memoryAddress, byteValue);
    counter = 3;
    do {
        byteValue = 0;
        eeprom.Read(memoryAddress, &byteValue);
        std::cout << "byteValue:0x" << std::hex << std::setw(2) << std::setfill('0') << (int)byteValue << "\r" << std::endl;
    } while(counter++ < maxCounter);
//    while(1);
    // Test short read/write
    short shortValue = 0xdead;
    eeprom.Write(memoryAddress + sizeof(unsigned char), shortValue);
    counter = 4;
    do {
        shortValue = 0;
        eeprom.Read(memoryAddress + sizeof(unsigned char), &shortValue);
        std::cout << "shortValue:0x" << std::hex << std::setw(4) << std::setfill('0') << (short)shortValue << "\r" << std::endl;
    } while(counter++ < maxCounter);
    eeprom.Read(memoryAddress, &byteValue);
    std::cout << "byteValue:0x" << std::hex << std::setw(2) << std::setfill('0') << (int)byteValue << "\r" << std::endl;
//    while(1);
    // Test integer read/write
    int intValue = 0xcafedeca;
    eeprom.Write(memoryAddress + sizeof(unsigned char) + sizeof(short), intValue);
    counter = 4;
    do {
        intValue = 0;
        eeprom.Read(memoryAddress + sizeof(unsigned char) + sizeof(short), &intValue);
        std::cout << "intValue:0x" << std::hex << std::setw(8) << std::setfill('0') << intValue << " - " << std::dec << intValue << "\r" << std::endl;
    } while(counter++ < maxCounter);
    eeprom.Read(memoryAddress, &byteValue);
    std::cout << "byteValue:0x" << std::hex << std::setw(2) << std::setfill('0') << (int)byteValue << "\r" << std::endl;
    eeprom.Read(memoryAddress + sizeof(unsigned char), &shortValue);
    std::cout << "shortValue:0x" << std::hex << std::setw(4) << std::setfill('0') << (short)shortValue << "\r" << std::endl;
//    while(1);
    // Test byte array read/write
    vector<unsigned char> bytesValue;
    bytesValue.push_back(0xff);
    bytesValue.push_back(0xfe);
    bytesValue.push_back(0xfd);
    bytesValue.push_back(0xfc);
    bytesValue.push_back(0xfa);
    bytesValue.push_back(0xf9);
    eeprom.Write(memoryAddress + sizeof(unsigned char) + sizeof(short) + sizeof(int), bytesValue);
    counter = 4;
    do {
        bytesValue.clear();
        eeprom.Read(memoryAddress + sizeof(unsigned char) + sizeof(short) + sizeof(int), bytesValue);
        for (int i = 0; i < bytesValue.size(); i++) { 
            std::cout << "bytesValue[" << i << "]=0x" << std::hex << std::setw(8) << std::setfill('0') << intValue << " - " << std::dec << intValue << "\r" << std::endl;
        }
    } while(counter++ < maxCounter);
    eeprom.Read(memoryAddress, &byteValue);
    std::cout << "byteValue:0x" << std::hex << std::setw(2) << std::setfill('0') << (int)byteValue << "\r" << std::endl;
    eeprom.Read(memoryAddress + sizeof(unsigned char), &shortValue);
    std::cout << "shortValue:0x" << std::hex << std::setw(4) << std::setfill('0') << (short)shortValue << "\r" << std::endl;
    bytesValue.clear();
    eeprom.Read(memoryAddress + sizeof(unsigned char) + sizeof(short) + sizeof(int), bytesValue);
    for (int i = 0; i < bytesValue.size(); i++) { 
        std::cout << "bytesValue[" << i << "]=0x" << std::hex << std::setw(8) << std::setfill('0') << intValue << " - " << std::dec << intValue << "\r" << std::endl;
    } // End of 'while' statement 
//    while(1);
    // Test string read/write
    std::string strValue("Test1234");
    eeprom.Write(memoryAddress + sizeof(unsigned char) + sizeof(short) + sizeof(int) + 6 * sizeof(unsigned char), strValue);
    counter = 4;
    do {
        strValue.clear();
        eeprom.Read(memoryAddress + sizeof(unsigned char) + sizeof(short) + sizeof(int) + 6 * sizeof(unsigned char), strValue);
        std::cout << "strValue: \"" << strValue << "\"\r" << std::endl;
    } while(counter++ < maxCounter);
    eeprom.Read(memoryAddress, &byteValue);
    std::cout << "byteValue:0x" << std::hex << std::setw(2) << std::setfill('0') << (int)byteValue << "\r" << std::endl;
    eeprom.Read(memoryAddress + sizeof(unsigned char), &shortValue);
    std::cout << "shortValue:0x" << std::hex << std::setw(4) << std::setfill('0') << (short)shortValue << "\r" << std::endl;
    bytesValue.clear();
    eeprom.Read(memoryAddress + sizeof(unsigned char) + sizeof(short) + sizeof(int), bytesValue);
    for (int i = 0; i < bytesValue.size(); i++) { 
        std::cout << "bytesValue[" << i << "]=0x" << std::hex << std::setw(8) << std::setfill('0') << intValue << " - " << std::dec << intValue << "\r" << std::endl;
    } // End of 'while' statement 
    strValue.clear();
    eeprom.Read(memoryAddress + sizeof(unsigned char) + sizeof(short) + sizeof(int) + 6 * sizeof(unsigned char), strValue);
    std::cout << "strValue: \"" << strValue << "\"\r" << std::endl;
    std::cout << "Done\r" << std::endl;
//    while(1);
    eeprom.DumpMemoryArea(memoryAddress, 64);
    while(1);
} // End of function main

void AvailableIndicator()
{
    g_availableLed = !g_availableLed;
} // End of AvailableIndicator

